function [p,a,d,deltain,starts,ends]=mkfindp(phase,delta,h,model);
% mkfindp.........finds ray parameter corrsponding to an epicentral distance IN SPHERICAL EARTH
%
% call: [p,a,d,deltain,starts,ends]=mkfindp(phase,delta,h,model);
%
%       phase: string containing seismic phase name like 'P', 'S', 'ScS', 'PKPdf', etc.\
%              Phase names are case sensitive!
%       delta: epicentral distance [deg]
%              might be a vector of epicentral distances
%           h: focal depth [km]
%          model: A structure describing the velocity distribution.
%                 The structure is expected to have the following fields:
%                 model.z: depth [km below surface] (array)
%                 model.vp: Vp [km/s] (array)
%                 model.vs: Vs [km/s] (array)
%                 model.rho: rho [g/ccm] (array)
%                 model.qp: Qp (array)
%                 model.qs: Qs (array)
%                 model.conr: depth of conrad discontinuity
%                 model.moho: depth of moho
%                 model.d410: depth of Mantle-Transition Zone-discontinuity (the "410" on earth)
%                 model.d520: depth of olivine beta-gamma transition (the "520" on earth)
%                 model.d660: depth of lower mantle discontinuioty (the "660" on earth)
%                 model.cmb: depth of core mantle boundary
%                 model.icb: depth of inner core boundary
%                 model.dz: depths of additional discontinuities (array of numbers)
%                 model.dname: names of additional discontinuities (array of strings)
%                 model.rp: planetary radius
%                 model.name: name of model (string)
%                 such a structure can be obtained via MKREADND.
%
%
% result: p: ray parameter [sec/deg] at which the given PHASE appears at distance DELTA
%            NaN if no such ray parameter exists (shadow zones etc)
%         a: take off angle of the PHASE at the source
%            necessary to distinguish between upgoing and downgoing rays
%         d: distances at which the respective rays arrive [deg]
%            you need this since the ray parameter list might be longer than DELTA list.
%            (and there might be small differences between the input values and these
%             results)
%         deltain: list of delta values for which the result is valid
%                  p(i), a(i), d(i) are the results for input value deltain(i)
%         starts: all index values (index to p,a,d) at which a continuous piece
%                 of function d(a) begins
%         ends:   all index values (index to p,a,d) at which a continuous piece
%                 of function d(a) ends.
%
% iterative search procedure.
%
% Martin Knapmeyer, 24.04.2002, 05.07.2002, 23.09.2003, 25.09.2003
%                   return NaN if no solution: 14.10.2003
%                   return start, ends, improved cutting: 28.10.2003

%tic;



%%% initialize result
pres=[];
ares=[];
dres=[];
p=NaN;
a=[];
d=[];
deltain=[];


%%% some constants
radian=pi/180;
epsilondeg=0.01; % epsilon used in delta-search [deg]


%%% prepare table of delta as function of p
dangle=0.1; % angle increment. MUST BE THAT SMALL - OTHERWISE YOU'LL MISS SOLUTIONS!
angles=mksmarttakeoff(phase,model,h,dangle); % angles at which source radiates


%%% determine velocity at source depth (needed for angle-ray parm conversion)
%%% if h is a discontinuity depth, there will be two velocioty values.
%%% vp(1) (or vs(1)) will be for above discontinuity, vp(2) (or vs(2)) for below.
focus=mkinterpmodel(model,h,'simple');
indy=find(focus.z==h);
vp=focus.vp(indy);
vs=focus.vs(indy);

%%% convert angles into ray parameters
anz=length(angles);
dist=zeros(1,anz);
p=dist;
for indy=1:anz
   %%% at a discontinuity, there will be two velocity values.
   %%% use above-value for angle>=90 and below-value for angle<90   
   if angles(indy)>=90
      whichangle=1;
   else
      whichangle=min(2,length(vp));
   end; % if angle
   p(indy)=mkangle2rayp(phase,h,angles(indy),vp(whichangle),vs(whichangle),model.rp);
   %%% now we have ray parameters to compute a distance from
   [dist(indy),rx,rz]=mkx4p(phase,h,p(indy),model,angles(indy));
end; % for angle
%%% now we have calculated p as function of angle and dist as function of p.

%toc

%%% the next task is to find any value of p for which dist(p) has the value
%%% given in input parameter DELTA. That's hard, since delta(p) is not smooth,
%%% not monotonic and not continuous.
%%% we have to cut delta(p) into continuous pieces and handle each of these pieces
%%% separately.
allreals=find((~isinf(dist))&(~isnan(dist)));
if ~isempty(allreals)
   lastreal=allreals(end);
	firstreal=allreals(1); % index of the first non-NaN-non-inf-Element of dist
	if firstreal==1
	   starts=[firstreal find(diff((~isnan(dist))&(~isinf(dist)))>0)+1]; % indexes where continuous pieces begin
	else
	   starts=[find(diff((~isnan(dist))&(~isinf(dist)))>0)+1];
	end; % if firstreal==1
	ends=find(diff((isnan(dist))&(isinf(dist)))>0); % indexes where continuous pieces end
	if length(ends)<length(starts)
       %%% the above procedure to find ends does not find the last element of DIST as end.
       %%% if the last continuous piece does not end before the last element, we have to 
       %%% add that to the list.
       ends=[ends lastreal]; %length(dist)];
	end; % if length(ends)
	
	
	%%% MK28102003
	%%% no we have identified all holes of function dist(p). But there might be
	%%% jumps in the function, which are not identified by the above procedure.
	%%% We therefore have to search for jumps in each of the pieces identified
	%%% up to now. Their begin and end indexes will be added to STARTS and ENDS.
	anz=length(starts);
	newstarts=[];
	newends=[];
	for i=1:anz
	    %%% mkfindjump finds the index of the last element before a jump
	    indexrange=starts(i):ends(i);
	    if ~isempty(indexrange)
	       jumps=mkfindjump(angles(indexrange),dist(indexrange));
	       jumps=jumps+(indexrange(1)-1); % make it an index into the complete array
	       additionalstarts=jumps+1;
	       additionalends=jumps;
	       newstarts=[newstarts starts(i) additionalstarts additionalends];
	       newends=[newends ends(i) additionalends additionalstarts];
	    end; % if ~isempty(indexrange)
	end; % for i
	starts=newstarts;
	ends=newends;
	starts=sort(starts);
	ends=sort(ends);
	
	%%% plot scan result as function of takeoff angle
	%clf; plot(angles,dist,'.-'); hold on; plot(angles(starts),dist(starts),'r>'); plot(angles(ends),dist(ends),'r<'); hold off; grid on
	%%% plot scan result as function of ray parameter
	%clf; plot(p,dist,'.-'); hold on; plot(p(starts),dist(starts),'r>'); plot(p(ends),dist(ends),'r<'); hold off; grid on
	
	%%% now we can search for those p for which dist(p)=DELTA.
	%%% that means: find zeros of mkx4p(phase,h,p(indy),vp,vs,r,rp)-delta for each piece
	%%% zeros are points at which y=0 is touched or crossed.
	%%% since DELTA might be a vector, we loop over all elements of DELTA.
   deltalen=length(delta);
   for deltacnt=1:deltalen
	   anz=length(starts);
	   for i=1:anz
          %%% search in the following piece
          distpiece=dist(starts(i):ends(i))-delta(deltacnt);
          ppiece=p(starts(i):ends(i));
          anglepiece=angles(starts(i):ends(i));
          piecelen=length(distpiece); % MK19112003, was: ends(i)-starts(i);
          %%% find left an right limits between zeros must be
          %%% (sorry for trickyness)
          indies=find([diff(sign(distpiece)) 0]~=0); % the last index before crossing y=0
          indies=[indies find(abs(distpiece)<epsilondeg)]; % add 0-touches to the list
          indies=unique(indies); % might be non-unique after the previous step!
       
          %%% we now have a list of ray parameters, which are the "last" value before
          %%% a zero. we go through this list and determine a better estimate of the
          %%% location of the zero.
          indyanz=length(indies);
          for indy=1:indyanz
             if (indies(indy)+1)<=piecelen
                %%% probable solution interval is fully covered by ray parm scan
                [pieceres,a,d]=mkfindzeros([ppiece(indies(indy)) ppiece(indies(indy)+1)],...
                             [distpiece(indies(indy)) distpiece(indies(indy)+1)]+delta(deltacnt),...
                             [anglepiece(indies(indy)) anglepiece(indies(indy)+1)],...
                             phase,h,model,delta(deltacnt),vp,vs);
             else
                %%% probable solution interval begins at last element of scan
                %%% searching between last two points may give a good approximation
                %%% and is probably stable - at least no out of range error occurs.
                %%% This could probably be improved. MK29.09.2003
                [pieceres,a,d]=mkfindzeros([ppiece(indies(indy)-1) ppiece(indies(indy))],...
                             [distpiece(indies(indy)-1) distpiece(indies(indy))]+delta(deltacnt),...
                             [anglepiece(indies(indy)-1) anglepiece(indies(indy))],...
                             phase,h,model,delta(deltacnt),vp,vs);
             end; % if (indies(indy)+1)<=piecelen
             if abs(d-delta(deltacnt))<epsilondeg
                  pres=[pres pieceres];
                  dres=[dres d];
                  ares=[ares a];
                  deltain=[deltain delta(deltacnt)];
             else % else added MK14102003
                  %%% if no ray parameter was found, return NaN!
             end; % if d-delta<epsilondeg
          end; % for indy
      end; % for i
	end; % for deltacnt
	
	
	
	%%% return result
	p=pres; %(find(~isnan(res)));
	if isempty(p)
	  p=NaN;
	end; % if isempty
	a=ares;
	d=dres; %(find(~isnan(res)));
else
   p=NaN; %[];
   a=NaN;
   d=NaN; %[];
end; % if ~isempty(allreal)


%%% finally, we have to construct the STARTS and ENDS lists for output.
%%% these range completely different from those determined after
%%% the continuous scan through all take off angles!
%%% we therefore have to search for values of a (output takeoff angles)
%%% which are closest to the start and end values determined above.
[asort,sorter]=sort(a); % sort a by size
newstarts=[];
newends=[];
anz=length(starts);
for indy=1:anz
    %%% a and angles are assumed to be ordered by size.
    %%% then the first element of indies will point to the smallest element of a
    %%% which is larger than angles(ends(indy)), 
    startindies=find(asort>=angles(starts(indy)));
    if ~isempty(startindies)
       newstarts=[newstarts startindies(1)];
    end; % if ~isempty(startindies)
    
    %%% a and angles are still assumed to be ordered by size.
    %%% then the last element of indies will point to the largest element of a
    %%% which is smaller than angles(ends(indy)), 
    endindies=find(asort<=angles(ends(indy)));
    if ~isempty(endindies)
       newends=[newends endindies(end)];
    end; % if ~isempty(endindies)
end; % for anz
%%% transform index values for ordering by a into index values for ordering by d
newstarts=sorter(newstarts);
newends=sorter(newends);

%%% index lists should be made unique
newstarts=unique(newstarts);
newends=unique(newends);



starts=newstarts;
ends=newends;


%%% now we're done.
%toctime=toc; disp(['MKFINDP: elapsed time ' mks2hms(toctime)]);


return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                 HELPER FUNCTIONS                                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [p,a,d]=mkfindzeros(prange,drange,arange,phase,h,model,delta,vp,vs);
% mkfindzero.........finds zeros of mkx4p(phase,h,p(indy),model)-delta
%
% call: [p,a,d]=mkfindzeros(prange,drange,arange,phase,h,model,delta,vp,vs);
%
%         prange: Two element vector defining the
%                 range of ray parameters in which to search
%                 It is assumed that exactly one zero lies between prange(1)
%                 and prange(2)
%         drange: epicentral distance (delta) values corresponding to prange values.
%                 used to make a first guess for the solution by linear interpolation
%         arange: take off angle range corresponding to PRANGE
%                 necessary to distinguish between upgoing and downgoing rays
%         vp: p velocity at source depth (may be two elements vector at discontinuities)
%         vs: s velocioty at source depth (may be two elements vector at discontinuities)
%
%         other input parms as in mkfindp.
%
% result: p: vector containing the ray parameter within PRANGE for which
%            mkx4p returns the DELTA desired by MKFINDP.
%         a: take off angle of the PHASE at the source
%            necessary to distinguish between upgoing and downgoing rays       
%         d: The DELTA that MKX4P returns for P.
%
%         p,a will be empty if no solution exists.
%
% the function actually minimizes abs(mkx4p()-delta) in order to find points
% where y=0 is not crossed but touched.
%
% Algorithm: starts at ARANGE(1) and increases a by some da. flips direction
% as soon as the target function starts running away from zero, and decreases da by some factor.
% This is an iterative process which should converge to the zeros of the target function.
% If there is more than one zero within the ARANGE-interval, it depends on their distribution
% which one is returned. So make sure that your intervals contain only one solution.
% To get a first guess for the soluition, linear interpolation between the borders of the intercval is used.
%
% Martin Knapmeyer, 07.05.2002


%%% initialize search
direct=1; % +1: increase p, -1: decrease p
da=(arange(2)-arange(1))/20; % initial stepwidth for increasing a. may be small since the first guess will be good.
epsilon=0.001; % "zero" is when abs()<epsilon
cnt=0; % counter for  iterations
evalcnt=0; % counter for function evaluations
maxcnt=50; % no more than MAXCNT iterations
currenta=interp1(drange,arange,delta);
if ~isnan(currenta)
   %%% interpolation found approx solution
   if currenta>=90
         whichangle=1;
      else
         whichangle=min(2,length(vp));
      end; % if angle
   currentp=mkangle2rayp(phase,h,currenta,vp(whichangle),vs(whichangle),model.rp);
   %disp(['MKFINDZEROS: prange:' num2str(prange) ' drange: ' num2str(drange) ' 1st guess: ' num2str(currentp)]);
   badness=abs(mkx4p(phase,h,currentp,model,currenta)-delta);
   evalcnt=evalcnt+1;
   oldbad=badness;
   done=0;
   while done==0
      if cnt>=maxcnt
         done=1; % give up
         %disp(['MKFINDZEROS: give up for ' num2str(delta)]);
      else
         %%% compute new function value
         oldbad=badness;
         d=mkx4p(phase,h,currentp,model,currenta);
         badness=abs(d-delta);
     
         evalcnt=evalcnt+1;
         
         if badness<epsilon
            done=1; % we're close enough!
         else
         
            %%% flip direction if necessary
            %%% interval boundary crossed?
            if (currenta<arange(1))|(currenta>arange(2))
               %disp('MKFINDZEROS: flip due to boundary crossing');
               direct=-direct;
               
            else
                %%% badness got worse?
                if (oldbad<badness)
                   %disp('MKFINDZEROS: flip due to badness increase!');
                   direct=-direct;
                   da=da/3;
                end; % if
            end; % if (currenta<arange(1))|(currenta>arange(2))
            
            %%% new CURRENTA
            currenta=currenta+direct*da;
            
            %%% new CURRENTP
            if currenta>=90
               whichangle=1;
            else
               whichangle=min(2,length(vp));
            end; % if angle
            currentp=mkangle2rayp(phase,h,currenta,vp(whichangle),vs(whichangle),model.rp);
           
            %%% increase iteration count
            cnt=cnt+1; % next iteration...
         end; % if badness else
      end; % if cnt>=maxcnt
   end; % while done
%disp(['MKFINDZEROS: ' int2str(evalcnt) ' evaluations in ' int2str(cnt) ' iterations.']);
else
   %%% interpolation did not find approx. solution - return empty
   currentp=[];
   currenta=[];
   d=[];
end; % if ~isempty(currenta)

% disp(['MKFINDZEROS: iteration count upon exit: ' int2str(cnt)]);
% if cnt==50
%    disp(cnt);
% end; % if cnt
%disp(['MKFINDZEROS: results: p=' num2str(currentp) ', a=' num2str(currenta) ', d=' num2str(d)]);

%%% return result
p=currentp;
a=currenta;
% d=d; % this is already done

return;